/*---------------------------------------------------------------
 *	micro:bitの1つのLED(左上のD2)を点滅(μT-Kernel 3.0用)
 *
 *	Copyright (C) 2022-2023 by T3 WG of TRON Forum
 *---------------------------------------------------------------*/
#include <tk/tkernel.h>
#include <tm/tmonitor.h>

// LED制御のためのGPIOの初期設定
LOCAL void led_init(void)
{
	// COL1..COL5とROW1..ROW5に接続されたGPIOピンを出力に設定(※A)
	out_w(GPIO(P0, PIN_CNF(28)), 1);	// GPIO P0.28(COL1に接続)
	out_w(GPIO(P0, PIN_CNF(11)), 1);	// GPIO P0.11(COL2に接続)
	out_w(GPIO(P0, PIN_CNF(31)), 1);	// GPIO P0.31(COL3に接続)
	out_w(GPIO(P1, PIN_CNF(05)), 1);	// GPIO P1.05(COL4に接続)
	out_w(GPIO(P0, PIN_CNF(30)), 1);	// GPIO P0.30(COL5に接続)
	out_w(GPIO(P0, PIN_CNF(21)), 1);	// GPIO P0.21(ROW1に接続)
	out_w(GPIO(P0, PIN_CNF(22)), 1);	// GPIO P0.22(ROW2に接続)
	out_w(GPIO(P0, PIN_CNF(15)), 1);	// GPIO P0.15(ROW3に接続)
	out_w(GPIO(P0, PIN_CNF(24)), 1);	// GPIO P0.24(ROW4に接続)
	out_w(GPIO(P0, PIN_CNF(19)), 1);	// GPIO P0.19(ROW5に接続)
}

// GPIOの特定のピンpinの出力をval(0または1)に設定(※B)
//	pinで指定された以外のピンの状態は変化しない
LOCAL void out_gpio_pin(UW port, UW pin, UW val)
{
	INT port_addr;

	if(port == 1){							// P1の場合
		if(val)
			port_addr = GPIO(P1, OUTSET);	// P1でvalが1の場合
		else							
			port_addr = GPIO(P1, OUTCLR);	// P1でvalが0の場合

	} else {								// P0の場合
		if(val)
			port_addr = GPIO(P0, OUTSET);	// P0でvalが1の場合
		else
			port_addr = GPIO(P0, OUTCLR);	// P0でvalが0の場合
	}
	out_w(port_addr, (1 << pin));			// 指定のピンに1を出力
}

// 1つのLED(左上のD2)を点滅するプログラム
EXPORT void usermain(void)
{
	UW gpio_p0_21 = 0;			// GPIO P0.21の出力、0または1
	UW gpio_p0_28 = 0;			// GPIO P0.28の出力、0または1

	led_init();

	out_gpio_pin(0, 28, 1);		// GPIO P0.28-COL1に1を出力(※C)
	out_gpio_pin(0, 11, 1);		// GPIO P0.11-COL2に1を出力(※C)
	out_gpio_pin(0, 31, 1);		// GPIO P0.31-COL3に1を出力(※C)
	out_gpio_pin(1,  5, 1);		// GPIO P1.05-COL4に1を出力(※C)
	out_gpio_pin(0, 30, 1);		// GPIO P0.30-COL5に1を出力(※C)

	for (;;) {					// 約1秒ごとに永久に繰り返し

		// GPIO P0.21とGPIO P0.28の出力値をコンソールに表示(※D)
		tm_printf("GPIO P0.21=%d(ROW1), GPIO P0.28=%d(COL1)\n",
				gpio_p0_21, gpio_p0_28);

		// GPIOへの出力(※E)
		out_gpio_pin(0, 21, gpio_p0_21);	// GPIO P0.21-ROW1にgpio_p0_21を出力
		out_gpio_pin(0, 28, gpio_p0_28);	// GPIO P0.28-COL1にgpio_p0_28を出力

		// gpio_p0_21とgpio_p0_28を更新して4通りの状態を順に生成
		// 	(gpio_p0_21,gpio_p0_28)が(0,0)→(1,0)→(0,1)→(1,1)→(0,0)...と変化
		if(++gpio_p0_21 >= 2){				// gpio_p0_21に1を加えて更新(※F)
			// gpio_p0_21が1から2に更新された場合
			gpio_p0_21 = 0;					// gpio_p0_21を0に戻す
			if(++gpio_p0_28 >= 2){			// gpio_p0_28に1を加えて更新
				// gpio_p0_28が1から2に更新された場合
				gpio_p0_28 = 0;				// gpio_p0_28を0に戻す
			}
		}
        tk_dly_tsk(1000);					// 1秒(1000ms)の間隔でループ
	}
}
